@@ -1,6 +1,6 @@
 package App::Greple::xlate::deepl;
 
-our $VERSION = "0.03";
+our $VERSION = "0.04";
 
 use v5.14;
 use warnings;
@@ -8,21 +8,22 @@ use Data::Dumper;
 
 use App::cdif::Command;
 
-our $lang_from = 'ORIGINAL';
-our $lang_to = 'JA';
+our $lang_from //= 'ORIGINAL';
+our $lang_to   //= 'JA';
 our $auth_key;
 our $max_length = 128 * 1024;
 
 sub xlate {
-    $DB::single = 1;
     state $deepl = App::cdif::Command->new;
     state $command = [ 'deepl', 'text',
 		       '--to' => $lang_to,
 		       $auth_key ? ('--auth-key' => $auth_key) : () ];
     my @from = map { /\n\z/ ? $_ : "$_\n" } @_;
     my $from = join '', @from;
+    my @count = map { int tr/\n/\n/ } @from;
     my $to = $deepl->command([@$command, $from])->update->data;
-    wantarray ? $to =~ /.+\n/g : $to;
+    my @out = $to =~ /.*\n/g;
+    map { join '', splice @out, 0, $_ } @count;
 }
 
 1;
